package javax.swing.text;

import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.Collections;
import java.util.Enumeration;
import java.util.LinkedHashMap;
import java.io.Serializable;

public class SimpleAttributeSet implements MutableAttributeSet, Serializable, Cloneable
{
    private static final long serialVersionUID = -6631553454711782652L;
    public static final AttributeSet EMPTY;
    private transient LinkedHashMap<Object, Object> table;
    
    public SimpleAttributeSet() {
        this.table = new LinkedHashMap(3);
    }
    
    public SimpleAttributeSet(AttributeSet set) {
        this.table = new LinkedHashMap(3);
        this.addAttributes(set);
    }
    
    public boolean isEmpty() {
        return this.table.isEmpty();
    }
    
    @Override
    public int getAttributeCount() {
        return this.table.size();
    }
    
    @Override
    public boolean isDefined(Object key) {
        return this.table.containsKey(key);
    }
    
    @Override
    public boolean isEqual(AttributeSet set) {
        return this.getAttributeCount() == set.getAttributeCount() && this.containsAttributes(set);
    }
    
    @Override
    public AttributeSet copyAttributes() {
        return (AttributeSet)this.clone();
    }
    
    @Override
    public Enumeration<?> getAttributeNames() {
        return Collections.enumeration(this.table.keySet());
    }
    
    @Override
    public Object getAttribute(Object key) {
        Object o = this.table.get(key);
        if (o == null) {
            AttributeSet resolveParent = this.getResolveParent();
            if (resolveParent != null) {
                o = resolveParent.getAttribute(key);
            }
        }
        return o;
    }
    
    @Override
    public boolean containsAttribute(Object o, Object o2) {
        return o2.equals(this.getAttribute(o));
    }
    
    @Override
    public boolean containsAttributes(AttributeSet set) {
        boolean equals = true;
        Object nextElement;
        for (Enumeration attributeNames = set.getAttributeNames(); equals && attributeNames.hasMoreElements(); equals = set.getAttribute(nextElement).equals(this.getAttribute(nextElement))) {
            nextElement = attributeNames.nextElement();
        }
        return equals;
    }
    
    @Override
    public void addAttribute(Object key, Object value) {
        this.table.put(key, value);
    }
    
    @Override
    public void addAttributes(AttributeSet set) {
        Enumeration<Object> attributeNames = set.getAttributeNames();
        while (attributeNames.hasMoreElements()) {
            Object nextElement = attributeNames.nextElement();
            this.addAttribute(nextElement, set.getAttribute(nextElement));
        }
    }
    
    @Override
    public void removeAttribute(Object key) {
        this.table.remove(key);
    }
    
    @Override
    public void removeAttributes(Enumeration<?> enumeration) {
        while (enumeration.hasMoreElements()) {
            this.removeAttribute(enumeration.nextElement());
        }
    }
    
    @Override
    public void removeAttributes(AttributeSet set) {
        if (set == this) {
            this.table.clear();
        }
        else {
            Enumeration<Object> attributeNames = set.getAttributeNames();
            while (attributeNames.hasMoreElements()) {
                Object nextElement = attributeNames.nextElement();
                if (set.getAttribute(nextElement).equals(this.getAttribute(nextElement))) {
                    this.removeAttribute(nextElement);
                }
                else {
                    continue;
                }
            }
        }
    }
    
    @Override
    public AttributeSet getResolveParent() {
        return (AttributeSet)this.table.get(StyleConstants.ResolveAttribute);
    }
    
    @Override
    public void setResolveParent(AttributeSet set) {
        this.addAttribute(StyleConstants.ResolveAttribute, set);
    }
    
    public Object clone() {
        SimpleAttributeSet set;
        try {
            set = (SimpleAttributeSet)super.clone();
            set.table = (LinkedHashMap)this.table.clone();
        }
        catch (CloneNotSupportedException ex) {
            set = null;
        }
        return set;
    }
    
    @Override
    public int hashCode() {
        return this.table.hashCode();
    }
    
    @Override
    public boolean equals(Object o) {
        return this == o || (o instanceof AttributeSet && this.isEqual((AttributeSet)o));
    }
    
    @Override
    public String toString() {
        String s = "";
        Enumeration<Object> attributeNames = this.getAttributeNames();
        while (attributeNames.hasMoreElements()) {
            Object nextElement = attributeNames.nextElement();
            Object attribute = this.getAttribute(nextElement);
            if ((attribute instanceof AttributeSet)) {
                s = s + nextElement + "=**AttributeSet** ";
            }
            else {
                s = s + nextElement + "=" + attribute + " ";
            }
        }
        return s;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        StyleContext.writeAttributeSet(objectOutputStream, this);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException {
        objectInputStream.defaultReadObject();
        this.table = new LinkedHashMap(3);
        StyleContext.readAttributeSet(objectInputStream, this);
    }
    
    static {
        EMPTY = new SimpleAttributeSet.EmptyAttributeSet();
    }
    
    static class EmptyAttributeSet implements AttributeSet, Serializable
    {
        static final long serialVersionUID = -8714803568785904228L;
        
        @Override
        public int getAttributeCount() {
            return 0;
        }
        
        @Override
        public boolean isDefined(Object o) {
            return false;
        }
        
        @Override
        public boolean isEqual(AttributeSet set) {
            return set.getAttributeCount() == 0;
        }
        
        @Override
        public AttributeSet copyAttributes() {
            return this;
        }
        
        @Override
        public Object getAttribute(Object o) {
            return null;
        }
        
        @Override
        public Enumeration getAttributeNames() {
            return Collections.emptyEnumeration();
        }
        
        @Override
        public boolean containsAttribute(Object o, Object o2) {
            return false;
        }
        
        @Override
        public boolean containsAttributes(AttributeSet set) {
            return set.getAttributeCount() == 0;
        }
        
        @Override
        public AttributeSet getResolveParent() {
            return null;
        }
        
        @Override
        public boolean equals(Object o) {
            return this == o || (o instanceof AttributeSet && ((AttributeSet)o).getAttributeCount() == 0);
        }
        
        @Override
        public int hashCode() {
            return 0;
        }
    }
}
